#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['qgrabber', 'qgrabber.pygrabber']

package_data = \
{'': ['*']}

install_requires = \
['aio-wx-widgets>=0.0.4', 'comtypes>=1.1.7', 'pyzbar', 'numpy', 'aiosubpub']

extras_require = \
{'dev': ['pylint', 'nox', 'pytest-cov', 'coverage[toml]>=5.0.3', 'pre-commit'],
 'test': ['pytest', 'pytest-asyncio', 'pytest-mock']}

setup(name='qgrabber',
      version='0.0.1',
      description='Scan QR codes using your webcam, pyzbar and wxpython. No usage of opencv',
      author='Sander Teunissen',
      author_email='no_mail@nomail.com',
      url='https://github.com/sander76/qrabber',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
